#ifndef __MTRAND__
#define __MTRAND__

#ifndef __GNUC__
	#pragma warning(disable : 4514) // unreferenced inline function removed
#endif

/********************************************************************************
*
* MTRandClass - 'Mersenne Twister' random number generator. (mt19987)
*
* 2-4 times faster than standard C rand, much more random than standard C rand
*
********************************************************************************/
class MTRandClass
{
private:	
	enum { N = 624 };               // length of state vector
	enum { M = 397 };               // a period parameter
	enum { K = 0x9908B0DFU };       // a magic constant
	

	unsigned long   seed;
	unsigned long * next;
	long            left;  
	unsigned long   state[N+1];
	
	
	unsigned long ReloadMT(void);
	
public:
	/*******************************************************************************
	*
	*  Initialization and seed functions
	*
	*******************************************************************************/
	inline MTRandClass(void) 
	{
		left = -1;
		next = 0;
		
		RandomSeed(0);
	}
	
	
	inline MTRandClass(unsigned long rand_seed) 
	{ 
		left = -1; 
		next = 0; 
		
		RandomSeed(rand_seed);
	}
	

	void RandomSeed(unsigned long rand_seed);
	


	/*******************************************************************************
	*
	*  Unsigned integer functions
	*
	*******************************************************************************/	
	inline unsigned long RandomUnsignedLong(void) 
	{
		if(--left < 0)
			return(ReloadMT());

		register unsigned long y;

		y  = *next++;
		y ^= (y >> 11);
		y ^= (y <<  7) & 0x9D2C5680U;
		y ^= (y << 15) & 0xEFC60000U;
		return(y ^ (y >> 18));
	}


	inline unsigned long RandomUnsignedLong(unsigned long limit)
	{
		unsigned long tmp = RandomUnsignedLong();
		unsigned long retval;
#ifdef __GNUC__
		__asm__ (
			"movl %1, %%eax\n\t"
			"movl %2, %%edx\n\t"
			"movl $0xFFFFFFFF, %%ecx\n\t"
			"mull %%edx\n\t"
			"divl %%ecx\n\t"
			"movl %%eax, %0\n\t"
			:"=r"(retval)
			:"rm"(limit), "rm"(tmp)
			:"eax","edx","ecx"
		);
#else
		__asm {
			mov eax, limit
			mov edx, tmp
			mov ecx, 0xFFFFFFFFU
			mul edx
			div ecx
			mov retval, eax
		}
#endif
		return(retval);
	}
	

	inline unsigned long RandomUnsignedLong(unsigned long lower_limit, unsigned long upper_limit)
	{
		register unsigned long limit;
		
		if( lower_limit > upper_limit ) {
			limit       = lower_limit - upper_limit;
			lower_limit = upper_limit;
		}
		else if( lower_limit == upper_limit )
			return(lower_limit);
		else {
			limit = upper_limit - lower_limit;
		}
			
		return( lower_limit + RandomUnsignedLong(limit) );
	}

	
	/*******************************************************************************
	*
	*  Signed integer functions
	*
	*******************************************************************************/
	inline long RandomLong(void) 
	{
		return( static_cast <long> ( RandomUnsignedLong( ) ) );
	}	
	
	
	inline long RandomLong(long lower_limit, long upper_limit)
	{
		register unsigned long limit;
		
		if( lower_limit > upper_limit ) {
			limit       = lower_limit - upper_limit;
			lower_limit = upper_limit;
		}
		else if( lower_limit == upper_limit )
			return(lower_limit);
		else {
			limit = upper_limit - lower_limit;
		}
			
		return( lower_limit + RandomUnsignedLong(limit) );
	}


	inline long RandomLong( long limit )
	{
		return( RandomLong(0, limit) );
	}
	
	
	/*******************************************************************************
	*
	*  64bit floating point functions
	*
	*******************************************************************************/
	inline double RandomDouble(void)
	{
		return((double)RandomUnsignedLong() * 2.3283064370807974e-10);
	}


	inline double RandomDouble(double limit)
	{
		return(RandomDouble() * limit);
	}

	
	inline double RandomDouble(double lower_limit, double upper_limit)
	{
		return(RandomDouble(upper_limit - lower_limit) + lower_limit);
	}


	/*******************************************************************************
	*
	*  32bit floating point functions
	*
	*******************************************************************************/		
	inline float RandomFloat(void)
	{
		return((float)RandomUnsignedLong() * (float)2.3283064365386963e-10f);
	}
	
	
	inline float RandomFloat(float limit)
	{
		return(RandomFloat() * limit);
	}
	
		
	inline float RandomFloat(float lower_limit, float upper_limit)
	{
		return(RandomFloat(upper_limit - lower_limit) + lower_limit);
	}
};


#endif /*__MTRAND__*/
